@@ -3,7 +3,7 @@ package Example::Schema::Result::Person;
 use base 'Example::Schema::Result';
 
 __PACKAGE__->table("person");
-__PACKAGE__->load_components(qw/EncodedColumn Valiant::Result/);
+__PACKAGE__->load_components(qw/EncodedColumn/);
 
 __PACKAGE__->add_columns(
   id => { data_type => 'bigint', is_nullable => 0, is_auto_increment => 1 },
@@ -34,7 +34,7 @@ __PACKAGE__->validates(last_name => (presence=>1, length=>[2,48]));
 __PACKAGE__->validates(credit_cards => (result_set=>+{validations=>1, min=>2, max=>4}, on=>'profile' ));
 __PACKAGE__->validates(person_roles => (result_set=>+{validations=>1, min=>1}, on=>'profile' ));
 __PACKAGE__->validates(profile => (result=>+{validations=>1}, allow_blank=>1 ));
-
+__PACKAGE__->accept_nested_for('person_roles');
 __PACKAGE__->set_primary_key("id");
 __PACKAGE__->add_unique_constraint(['username']);
 
